/* eslint-disable @typescript-eslint/ban-types */
import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
class ByteAConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        if (context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        const [argument] = context.data, allowedTypes = [ParsedType.string, ParsedType.object, ParsedType.Buffer], parsedType = getParsedType(argument);
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case ParsedType.object:
                return this._parseObject(context, argument);
            case ParsedType.Buffer:
                return OK(new ByteAClass(argument));
            default:
                //* Stringify the argument to make sure it's a string
                return this._parseString(`${argument}`);
        }
    }
    _parseString(argument) {
        if (/^\\x/.test(argument))
            return OK(new ByteAClass(Buffer.from(argument.slice(2), "hex")));
        return this._parseBinaryString(argument);
    }
    _parseBinaryString(argument) {
        let binary = "", index = 0;
        while (index < argument.length) {
            if (argument[index] === "\\") {
                if (/[0-7]{3}/.test(argument.slice(index + 1, index + 4))) {
                    binary += String.fromCodePoint(Number.parseInt(argument.slice(index + 1, index + 4), 8));
                    index += 4;
                }
                else {
                    let backslashes = 1;
                    while (index + backslashes < argument.length && argument[index + backslashes] === "\\")
                        backslashes++;
                    for (let k = 0; k < Math.floor(backslashes / 2); ++k)
                        binary += "\\";
                    index += Math.floor(backslashes / 2) * 2;
                }
            }
            else {
                binary += argument[index];
                ++index;
            }
        }
        return OK(new ByteAClass(Buffer.from(binary, "binary")));
    }
    _parseObject(context, argument) {
        if (this.isByteA(argument))
            return OK(new ByteAClass(argument.value));
        const parsedObject = hasKeys(argument, [["value", "Buffer"]]);
        if (parsedObject.success)
            return OK(new ByteAClass(parsedObject.obj.value));
        switch (true) {
            case parsedObject.otherKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "unrecognized_keys",
                    keys: parsedObject.otherKeys,
                });
                break;
            case parsedObject.missingKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "missing_keys",
                    keys: parsedObject.missingKeys,
                });
                break;
            case parsedObject.invalidKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "invalid_key_type",
                    ...parsedObject.invalidKeys[0],
                });
                break;
        }
        return INVALID;
    }
    isByteA(object) {
        return object instanceof ByteAClass;
    }
}
const ByteA = new ByteAConstructorClass();
class ByteAClass extends PgTPBase {
    constructor(_bytea) {
        super();
        this._bytea = _bytea;
    }
    _equals(input) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = ByteA.safeFrom(...input.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(input, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return `\\x${this._bytea.toString("hex")}`;
    }
    toBuffer() {
        return this._bytea;
    }
    toJSON() {
        return {
            value: this._bytea,
        };
    }
    get bytea() {
        return this._bytea;
    }
    set bytea(bytea) {
        const parsed = ByteA.safeFrom(bytea);
        if (parsed.success)
            this._bytea = parsed.data.toBuffer();
        else
            throw parsed.error;
    }
    get value() {
        return this._bytea;
    }
    set value(buffer) {
        const parsed = ByteA.safeFrom(buffer);
        if (parsed.success)
            this._bytea = parsed.data.toBuffer();
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(bytea) {
        const parsed = ByteA.safeFrom(bytea);
        if (parsed.success)
            this._bytea = parsed.data.toBuffer();
        else
            throw parsed.error;
    }
}
export { ByteA };
//# sourceMappingURL=ByteA.js.map