import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { throwPgTPError } from "../../util/throwPgTPError.js";
import { Character } from "../Character/Character.js";
import { CharacterVarying } from "../Character/CharacterVarying.js";
import { Name } from "../Character/Name.js";
import { Text } from "../Character/Text.js";
import { Int2 } from "../Numeric/Int2.js";
import { Int4 } from "../Numeric/Int4.js";
import { Int8 } from "../Numeric/Int8.js";
import { OID } from "../ObjectIdentifier/OID.js";
import { BitVarying } from "./BitVarying.js";
class BitConstructorClass extends PgTPConstructorBase {
    constructor(_n = 1) {
        super();
        this._n = _n;
        const allowedTypes = [ParsedType.number, ParsedType.infinity], parsedType = getParsedType(_n);
        if (!isOneOf(allowedTypes, parsedType)) {
            throwPgTPError({
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
        }
        if (_n < 1) {
            throwPgTPError({
                code: "too_small",
                type: "number",
                minimum: 1,
                inclusive: true,
                received: _n,
            });
        }
        //If limit is not a whole number
        if (_n % 1 !== 0 && _n !== Number.POSITIVE_INFINITY) {
            throwPgTPError({
                code: "not_whole",
                received: _n,
            });
        }
    }
    _parse(context) {
        if (context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        const [argument] = context.data, allowedTypes = [ParsedType.number, ParsedType.string, ParsedType.object], parsedType = getParsedType(argument);
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case ParsedType.object:
                return this._parseObject(context, argument);
            case ParsedType.number:
                return this._parseNumber(context, argument);
            default:
                return this._parseString(context, argument);
        }
    }
    _parseString(context, argument) {
        // Only allow 0s and 1s
        if (!/^[01]+$/.test(argument)) {
            this.setIssueForContext(context, {
                code: "invalid_string",
                expected: "LIKE 010101",
                received: argument,
            });
            return INVALID;
        }
        // Make sure the bit is the correct length
        if (argument.length > this._n) {
            this.setIssueForContext(context, {
                code: "invalid_n_length",
                maximum: this._n,
                received: argument.length,
                input: argument,
            });
            return INVALID;
        }
        // Truncate or zero-pad on the right to be exactly n bits
        if (argument.length < this._n && this._n !== Number.POSITIVE_INFINITY)
            argument = argument.padEnd(this._n, "0");
        return OK(new BitClass(argument, this._n));
    }
    _parseNumber(context, argument) {
        // Make sure the number is a whole number
        if (argument % 1 !== 0) {
            this.setIssueForContext(context, {
                code: "not_whole",
                received: argument,
            });
            return INVALID;
        }
        // Make sure the number is positive
        if (argument < 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                type: "number",
                minimum: 0,
                exact: true,
                received: argument,
            });
            return INVALID;
        }
        // Stringify the number and parse it as a string, >>> 0 is to double check that it's a whole number
        const string = (argument >>> 0).toString(2);
        return this._parseString(context, string);
    }
    _parseObject(context, argument) {
        if (this.isAnyBit(argument)) {
            // Make sure the bit is the correct length
            if (argument.n > this._n) {
                this.setIssueForContext(context, {
                    code: "invalid_n_length",
                    maximum: this._n,
                    received: argument.n,
                    input: argument.value,
                });
                return INVALID;
            }
            return OK(new BitClass(argument.value, this._n));
        }
        if (BitVarying.isAnyBitVarying(argument))
            return this._parseString(context, argument.value);
        if (Character.isAnyCharacter(argument))
            return this._parseString(context, argument.value);
        if (CharacterVarying.isAnyCharacterVarying(argument))
            return this._parseString(context, argument.value);
        if (Name.isName(argument))
            return this._parseString(context, argument.value);
        if (Text.isText(argument))
            return this._parseString(context, argument.value);
        if (Int2.isInt2(argument))
            return this._parseNumber(context, argument.value);
        if (Int4.isInt4(argument))
            return this._parseNumber(context, argument.value);
        if (Int8.isInt8(argument))
            return this._parseString(context, argument.toString());
        if (OID.isOID(argument))
            return this._parseNumber(context, argument.value);
        const parsedObject = hasKeys(argument, [["value", "string"]]);
        if (parsedObject.success)
            return this._parseString(context, parsedObject.obj.value);
        switch (true) {
            case parsedObject.otherKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "unrecognized_keys",
                    keys: parsedObject.otherKeys,
                });
                break;
            case parsedObject.missingKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "missing_keys",
                    keys: parsedObject.missingKeys,
                });
                break;
            case parsedObject.invalidKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "invalid_key_type",
                    ...parsedObject.invalidKeys[0],
                });
                break;
        }
        return INVALID;
    }
    isBit(object, n) {
        if (typeof n === "number")
            return this.setN(n).isBit(object);
        return object instanceof BitClass && object.n === this._n;
    }
    isAnyBit(object) {
        return object instanceof BitClass;
    }
    setN(n) {
        return new BitConstructorClass(n);
    }
    get n() {
        return this._n;
    }
}
const Bit = new BitConstructorClass(1);
class BitClass extends PgTPBase {
    constructor(_bit, _n = 1) {
        super();
        this._bit = _bit;
        this._n = _n;
    }
    _equals(input) {
        const parsed = new BitConstructorClass(this._n).safeFrom(...input.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(input, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return this._bit;
    }
    toNumber() {
        return Number.parseInt(this._bit, 2);
    }
    toJSON() {
        return {
            value: this._bit.toString(),
        };
    }
    get bit() {
        return this._bit;
    }
    set bit(bit) {
        const parsed = new BitConstructorClass(this._n).safeFrom(bit);
        if (parsed.success)
            this._bit = parsed.data.toString();
        else
            throw parsed.error;
    }
    get value() {
        return this._bit;
    }
    set value(bit) {
        const parsed = new BitConstructorClass(this._n).safeFrom(bit);
        if (parsed.success)
            this._bit = parsed.data.toString();
        else
            throw parsed.error;
    }
    get postgres() {
        return this._bit;
    }
    set postgres(bit) {
        const parsed = new BitConstructorClass(this._n).safeFrom(bit);
        if (parsed.success)
            this._bit = parsed.data.toString();
        else
            throw parsed.error;
    }
    get n() {
        return this._n;
    }
}
export { Bit };
//# sourceMappingURL=Bit.js.map