/* eslint-disable @typescript-eslint/ban-types */
import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
class BooleanConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        if (context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        const [argument] = context.data, allowedTypes = [ParsedType.number, ParsedType.string, ParsedType.object, ParsedType.boolean], parsedType = getParsedType(argument);
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case ParsedType.object:
                return this._parseObject(context, argument);
            default:
                //* Stringify the argument to make sure it's a string
                return this._parseString(context, `${argument}`);
        }
    }
    _parseString(context, argument) {
        //* Make sure the argument is lowercase and remove all whitespace
        argument = argument.toLowerCase().replaceAll(/\s/g, "");
        const trueValues = ["true", "t", "yes", "y", "1", "on"], falseValues = ["false", "f", "no", "n", "0", "off", "of"];
        if (trueValues.includes(argument))
            return OK(new BooleanClass(true));
        if (falseValues.includes(argument))
            return OK(new BooleanClass(false));
        this.setIssueForContext(context, {
            code: "invalid_string",
            expected: [...trueValues, ...falseValues],
            received: argument,
        });
        return INVALID;
    }
    _parseObject(context, argument) {
        if (this.isBoolean(argument))
            return OK(new BooleanClass(argument.value));
        const parsedObject = hasKeys(argument, [["value", "boolean"]]);
        if (parsedObject.success)
            return OK(new BooleanClass(parsedObject.obj.value));
        switch (true) {
            case parsedObject.otherKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "unrecognized_keys",
                    keys: parsedObject.otherKeys,
                });
                break;
            case parsedObject.missingKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "missing_keys",
                    keys: parsedObject.missingKeys,
                });
                break;
            case parsedObject.invalidKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "invalid_key_type",
                    ...parsedObject.invalidKeys[0],
                });
                break;
        }
        return INVALID;
    }
    isBoolean(object) {
        return object instanceof BooleanClass;
    }
}
const Boolean = new BooleanConstructorClass();
class BooleanClass extends PgTPBase {
    constructor(_boolean) {
        super();
        this._boolean = _boolean;
    }
    _equals(input) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = Boolean.safeFrom(...input.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toBoolean() === this.toBoolean(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(input, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return this._boolean.toString();
    }
    toNumber() {
        return this._boolean ? 1 : 0;
    }
    toBoolean() {
        return this._boolean;
    }
    toJSON() {
        return {
            value: this._boolean,
        };
    }
    get boolean() {
        return this._boolean;
    }
    set boolean(bool) {
        const parsed = Boolean.safeFrom(bool);
        if (parsed.success)
            this._boolean = parsed.data.toBoolean();
        else
            throw parsed.error;
    }
    get value() {
        return this._boolean;
    }
    set value(bool) {
        const parsed = Boolean.safeFrom(bool);
        if (parsed.success)
            this._boolean = parsed.data.toBoolean();
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(bool) {
        const parsed = Boolean.safeFrom(bool);
        if (parsed.success)
            this._boolean = parsed.data.toBoolean();
        else
            throw parsed.error;
    }
}
export { Boolean };
//# sourceMappingURL=Boolean.js.map