import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { throwPgTPError } from "../../util/throwPgTPError.js";
import { Bit } from "../BitString/Bit.js";
import { BitVarying } from "../BitString/BitVarying.js";
import { Int2 } from "../Numeric/Int2.js";
import { Int4 } from "../Numeric/Int4.js";
import { Int8 } from "../Numeric/Int8.js";
import { OID } from "../ObjectIdentifier/OID.js";
import { UUID } from "../UUID/UUID.js";
import { CharacterVarying } from "./CharacterVarying.js";
import { Name } from "./Name.js";
import { Text } from "./Text.js";
class CharacterConstructorClass extends PgTPConstructorBase {
    constructor(_n = 1) {
        super();
        this._n = _n;
        const allowedTypes = [ParsedType.number, ParsedType.infinity], parsedType = getParsedType(_n);
        if (!isOneOf(allowedTypes, parsedType)) {
            throwPgTPError({
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
        }
        if (_n < 1) {
            throwPgTPError({
                code: "too_small",
                type: "number",
                minimum: 1,
                inclusive: true,
                received: _n,
            });
        }
        if (_n > 10485760 && _n !== Number.POSITIVE_INFINITY) {
            throwPgTPError({
                code: "too_big",
                type: "number",
                maximum: 10485760,
                inclusive: true,
                received: _n,
            });
        }
        if (_n % 1 !== 0 && _n !== Number.POSITIVE_INFINITY) {
            //If limit is not a whole number
            throwPgTPError({
                code: "not_whole",
                received: _n,
            });
        }
    }
    _parse(context) {
        if (context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        const [argument] = context.data, allowedTypes = [ParsedType.string, ParsedType.object], parsedType = getParsedType(argument);
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case ParsedType.object:
                return this._parseObject(context, argument);
            default:
                return this._parseString(context, argument);
        }
    }
    _parseString(context, argument) {
        // Make sure the character is the correct length
        if (argument.length > this._n) {
            this.setIssueForContext(context, {
                code: "invalid_n_length",
                maximum: this._n,
                received: argument.length,
                input: argument,
            });
            return INVALID;
        }
        // Truncate or space-pad on the right to be exactly n characters
        if (argument.length < this._n && this._n !== Number.POSITIVE_INFINITY)
            argument = argument.padEnd(this._n, " ");
        return OK(new CharacterClass(argument, this._n));
    }
    _parseObject(context, argument) {
        if (this.isAnyCharacter(argument)) {
            // Make sure the character is the correct length
            if (argument.n > this._n) {
                this.setIssueForContext(context, {
                    code: "invalid_n_length",
                    maximum: this._n,
                    received: argument.n,
                    input: argument.value,
                });
                return INVALID;
            }
            return OK(new CharacterClass(argument.value, this._n));
        }
        if (Bit.isAnyBit(argument))
            return this._parseString(context, argument.value);
        if (BitVarying.isAnyBitVarying(argument))
            return this._parseString(context, argument.value);
        if (CharacterVarying.isAnyCharacterVarying(argument))
            return this._parseString(context, argument.value);
        if (Name.isName(argument))
            return this._parseString(context, argument.value);
        if (Text.isText(argument))
            return this._parseString(context, argument.value);
        if (Int2.isInt2(argument))
            return this._parseString(context, argument.toString());
        if (Int4.isInt4(argument))
            return this._parseString(context, argument.toString());
        if (Int8.isInt8(argument))
            return this._parseString(context, argument.toString());
        if (OID.isOID(argument))
            return this._parseString(context, argument.toString());
        if (UUID.isUUID(argument))
            return this._parseString(context, argument.value);
        const parsedObject = hasKeys(argument, [["value", "string"]]);
        if (parsedObject.success)
            return this._parseString(context, parsedObject.obj.value);
        switch (true) {
            case parsedObject.otherKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "unrecognized_keys",
                    keys: parsedObject.otherKeys,
                });
                break;
            case parsedObject.missingKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "missing_keys",
                    keys: parsedObject.missingKeys,
                });
                break;
            case parsedObject.invalidKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "invalid_key_type",
                    ...parsedObject.invalidKeys[0],
                });
                break;
        }
        return INVALID;
    }
    isCharacter(object, n) {
        if (typeof n === "number")
            return this.setN(n).isCharacter(object);
        return object instanceof CharacterClass && object.n === this._n;
    }
    isAnyCharacter(object) {
        return object instanceof CharacterClass;
    }
    setN(n) {
        return new CharacterConstructorClass(n);
    }
    get n() {
        return this._n;
    }
}
const Character = new CharacterConstructorClass(1);
class CharacterClass extends PgTPBase {
    constructor(_character, _n = 1) {
        super();
        this._character = _character;
        this._n = _n;
    }
    _equals(input) {
        const parsed = new CharacterConstructorClass(this._n).safeFrom(...input.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(input, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return this._character;
    }
    toJSON() {
        return {
            value: this._character,
        };
    }
    get character() {
        return this._character;
    }
    set character(character) {
        const parsed = new CharacterConstructorClass(this._n).safeFrom(character);
        if (parsed.success)
            this._character = parsed.data.toString();
        else
            throw parsed.error;
    }
    get value() {
        return this._character;
    }
    set value(character) {
        const parsed = new CharacterConstructorClass(this._n).safeFrom(character);
        if (parsed.success)
            this._character = parsed.data.toString();
        else
            throw parsed.error;
    }
    get postgres() {
        return this._character;
    }
    set postgres(character) {
        const parsed = new CharacterConstructorClass(this._n).safeFrom(character);
        if (parsed.success)
            this._character = parsed.data.toString();
        else
            throw parsed.error;
    }
    get n() {
        return this._n;
    }
}
export { Character };
//# sourceMappingURL=Character.js.map