import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { UUID } from "../UUID/UUID.js";
import { Character } from "./Character.js";
import { CharacterVarying } from "./CharacterVarying.js";
import { Text } from "./Text.js";
class NameConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        if (context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        const [argument] = context.data, allowedTypes = [ParsedType.string, ParsedType.object], parsedType = getParsedType(argument);
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case ParsedType.string:
                return this._parseString(context, argument);
            default:
                return this._parseObject(context, argument);
        }
    }
    _parseString(context, argument) {
        //* Calculate bytes used by string
        const bytes = Buffer.byteLength(argument, "utf8");
        //* Check if string is too long
        if (bytes > 64) {
            this.setIssueForContext(context, {
                code: "too_big",
                type: "bytes",
                maximum: 64,
                inclusive: true,
                received: bytes,
                input: argument,
            });
            return INVALID;
        }
        return OK(new NameClass(argument));
    }
    _parseObject(context, argument) {
        if (this.isName(argument))
            return OK(new NameClass(argument.value));
        if (Character.isAnyCharacter(argument))
            return this._parseString(context, argument.value);
        if (CharacterVarying.isAnyCharacterVarying(argument))
            return this._parseString(context, argument.value);
        if (Text.isText(argument))
            return this._parseString(context, argument.value);
        if (UUID.isUUID(argument))
            return this._parseString(context, argument.value);
        const parsedObject = hasKeys(argument, [["value", "string"]]);
        if (parsedObject.success)
            return this._parseString(context, parsedObject.obj.value);
        switch (true) {
            case parsedObject.otherKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "unrecognized_keys",
                    keys: parsedObject.otherKeys,
                });
                break;
            case parsedObject.missingKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "missing_keys",
                    keys: parsedObject.missingKeys,
                });
                break;
            case parsedObject.invalidKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "invalid_key_type",
                    ...parsedObject.invalidKeys[0],
                });
                break;
        }
        return INVALID;
    }
    isName(object) {
        return object instanceof NameClass;
    }
}
const Name = new NameConstructorClass();
class NameClass extends PgTPBase {
    constructor(_name) {
        super();
        this._name = _name;
    }
    _equals(context) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = Name.safeFrom(...context.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(context, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return this._name;
    }
    toJSON() {
        return {
            value: this._name,
        };
    }
    get name() {
        return this._name;
    }
    set name(name) {
        const parsed = Name.safeFrom(name);
        if (parsed.success)
            this._name = parsed.data.toString();
        else
            throw parsed.error;
    }
    get value() {
        return this._name;
    }
    set value(name) {
        const parsed = Name.safeFrom(name);
        if (parsed.success)
            this._name = parsed.data.toString();
        else
            throw parsed.error;
    }
    get postgres() {
        return this._name;
    }
    set postgres(name) {
        const parsed = Name.safeFrom(name);
        if (parsed.success)
            this._name = parsed.data.toString();
        else
            throw parsed.error;
    }
}
export { Name };
//# sourceMappingURL=Name.js.map