import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { DateTime } from "luxon";
import { isValidDate } from "../../util/isValidDate.js";
import { isValidDateTime } from "../../util/isValidDateTime.js";
import { pad } from "../../util/pad.js";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { throwPgTPError } from "../../util/throwPgTPError.js";
import { TimestampTZ } from "./TimestampTZ.js";
class TimeConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        const [argument, ...otherArguments] = context.data, allowedTypes = [ParsedType.number, ParsedType.string, ParsedType.object, ParsedType["globalThis.Date"], ParsedType["luxon.DateTime"]], parsedType = getParsedType(argument);
        if (parsedType !== ParsedType.number && context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case "string":
                return this._parseString(context, argument);
            case "number":
                return this._parseNumber(context, argument, otherArguments);
            default:
                return this._parseObject(context, argument);
        }
    }
    _parseString(context, argument) {
        const parsedTimestamp = TimestampTZ.safeFrom(argument);
        if (parsedTimestamp.success)
            return OK(new TimeClass(parsedTimestamp.data.hour, parsedTimestamp.data.minute, parsedTimestamp.data.second));
        this.setIssueForContext(context, {
            code: "invalid_string",
            received: argument,
            expected: "LIKE HH:MM:SS",
        });
        return INVALID;
    }
    _parseNumber(context, argument, otherArguments) {
        const totalLength = otherArguments.length + 1;
        if (totalLength === 1) {
            return this._parseObject(context, DateTime.fromMillis(argument, {
                zone: "UTC",
            }));
        }
        if (totalLength !== 3) {
            this.setIssueForContext(context, totalLength > 3
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 3,
                    exact: true,
                    received: totalLength,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 3,
                    exact: true,
                    received: totalLength,
                });
            return INVALID;
        }
        for (const otherArgument of otherArguments) {
            const allowedTypes = [ParsedType.number], parsedType = getParsedType(otherArgument);
            if (!isOneOf(allowedTypes, parsedType)) {
                this.setIssueForContext(context, {
                    code: "invalid_type",
                    expected: allowedTypes,
                    received: parsedType,
                });
                return INVALID;
            }
        }
        const [hour, minute, second] = [argument, ...otherArguments];
        return this._parseString(context, `${pad(hour)}:${pad(minute)}:${pad(second)}`);
    }
    _parseObject(context, argument) {
        // Should be [Time]
        if (Time.isTime(argument))
            return OK(new TimeClass(argument.hour, argument.minute, argument.second));
        // Should be [globalThis.Date]
        const jsDate = isValidDate(argument);
        if (jsDate.isOfSameType) {
            if (jsDate.isValid) {
                const [hour, minute, second] = [jsDate.data.getHours(), jsDate.data.getMinutes(), jsDate.data.getSeconds()];
                return this._parseString(context, `${pad(hour)}:${pad(minute)}:${pad(second)}`);
            }
            this.setIssueForContext(context, jsDate.error);
            return INVALID;
        }
        // Should be [luxon.DateTime]
        const luxonDate = isValidDateTime(argument);
        if (luxonDate.isOfSameType) {
            if (luxonDate.isValid) {
                const [hour, minute, second] = [luxonDate.data.hour, luxonDate.data.minute, luxonDate.data.second];
                return this._parseString(context, `${pad(hour)}:${pad(minute)}:${pad(second)}`);
            }
            this.setIssueForContext(context, luxonDate.error);
            return INVALID;
        }
        // Should be [TimeObject]
        const parsedObject = hasKeys(argument, [
            ["hour", "number"],
            ["minute", "number"],
            ["second", "number"],
        ]);
        if (!parsedObject.success) {
            switch (true) {
                case parsedObject.otherKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "unrecognized_keys",
                        keys: parsedObject.otherKeys,
                    });
                    break;
                case parsedObject.missingKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "missing_keys",
                        keys: parsedObject.missingKeys,
                    });
                    break;
                case parsedObject.invalidKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "invalid_key_type",
                        ...parsedObject.invalidKeys[0],
                    });
                    break;
            }
            return INVALID;
        }
        const { hour, minute, second } = parsedObject.obj;
        if (hour % 1 !== 0) {
            this.setIssueForContext(context, {
                code: "not_whole",
                received: hour,
            });
            return INVALID;
        }
        if (hour < 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: hour,
            });
            return INVALID;
        }
        if (hour > 23) {
            this.setIssueForContext(context, {
                code: "too_big",
                maximum: 23,
                type: "number",
                inclusive: true,
                received: hour,
            });
            return INVALID;
        }
        if (minute % 1 !== 0) {
            this.setIssueForContext(context, {
                code: "not_whole",
                received: minute,
            });
            return INVALID;
        }
        if (minute < 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: minute,
            });
            return INVALID;
        }
        if (minute > 59) {
            this.setIssueForContext(context, {
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
                received: minute,
            });
            return INVALID;
        }
        if (second < 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: second,
            });
            return INVALID;
        }
        if (second >= 60) {
            this.setIssueForContext(context, {
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
                received: second,
            });
            return INVALID;
        }
        return OK(new TimeClass(hour, minute, second));
    }
    isTime(object) {
        return object instanceof TimeClass;
    }
}
const Time = new TimeConstructorClass();
class TimeClass extends PgTPBase {
    constructor(_hour, _minute, _second) {
        super();
        this._hour = _hour;
        this._minute = _minute;
        this._second = _second;
    }
    _equals(context) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = Time.safeFrom(...context.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(context, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return `${pad(this._hour)}:${pad(this._minute)}:${pad(this._second)}`;
    }
    toNumber() {
        const todayUnix = DateTime.now().setZone("UTC").startOf("day").toMillis();
        return this.toDateTime("UTC").toMillis() - todayUnix;
    }
    toJSON() {
        return {
            hour: this._hour,
            minute: this._minute,
            second: this._second,
        };
    }
    toDateTime(zone) {
        return DateTime.fromObject({
            hour: this._hour,
            minute: this._minute,
            second: Math.trunc(this._second),
            millisecond: Math.round((this._second % 1) * 1000),
        }, { zone });
    }
    toJSDate(zone) {
        return this.toDateTime(zone).toJSDate();
    }
    get hour() {
        return this._hour;
    }
    set hour(hour) {
        const parsedType = getParsedType(hour);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (hour % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
                received: hour,
            });
        }
        if (hour < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: hour,
            });
        }
        if (hour > 23) {
            throwPgTPError({
                code: "too_big",
                maximum: 23,
                type: "number",
                inclusive: true,
                received: hour,
            });
        }
        this._hour = hour;
    }
    get minute() {
        return this._minute;
    }
    set minute(minute) {
        const parsedType = getParsedType(minute);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (minute % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
                received: minute,
            });
        }
        if (minute < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: minute,
            });
        }
        if (minute > 59) {
            throwPgTPError({
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
                received: minute,
            });
        }
        this._minute = minute;
    }
    get second() {
        return this._second;
    }
    set second(second) {
        const parsedType = getParsedType(second);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (second < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: second,
            });
        }
        if (second >= 60) {
            throwPgTPError({
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
                received: second,
            });
        }
        this._second = second;
    }
    get value() {
        return this.toString();
    }
    set value(time) {
        const parsed = Time.safeFrom(time);
        if (parsed.success) {
            this._hour = parsed.data.hour;
            this._minute = parsed.data.minute;
            this._second = parsed.data.second;
        }
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(time) {
        const parsed = Time.safeFrom(time);
        if (parsed.success) {
            this._hour = parsed.data.hour;
            this._minute = parsed.data.minute;
            this._second = parsed.data.second;
        }
        else
            throw parsed.error;
    }
}
export { Time };
//# sourceMappingURL=Time.js.map