import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { DateTime } from "luxon";
import { OffsetDirection } from "../../types/OffsetDirection.js";
import { isValidDate } from "../../util/isValidDate.js";
import { isValidDateTime } from "../../util/isValidDateTime.js";
import { pad } from "../../util/pad.js";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { throwPgTPError } from "../../util/throwPgTPError.js";
import { Date } from "./Date.js";
import { Time } from "./Time.js";
import { TimestampStyle, TimestampTZ } from "./TimestampTZ.js";
class TimestampConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        const [argument, ...otherArguments] = context.data, allowedTypes = [ParsedType.number, ParsedType.string, ParsedType.object, ParsedType["globalThis.Date"], ParsedType["luxon.DateTime"]], parsedType = getParsedType(argument);
        if (parsedType !== ParsedType.number && context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case "string":
                return this._parseString(context, argument);
            case "number":
                return this._parseNumber(context, argument, otherArguments);
            default:
                return this._parseObject(context, argument);
        }
    }
    _parseString(context, argument) {
        const parsedTimestamp = TimestampTZ.safeFrom(argument);
        if (parsedTimestamp.success) {
            return OK(new TimestampClass(parsedTimestamp.data.year, parsedTimestamp.data.month, parsedTimestamp.data.day, parsedTimestamp.data.hour, parsedTimestamp.data.minute, parsedTimestamp.data.second));
        }
        this.setIssueForContext(context, {
            code: "invalid_string",
            received: argument,
            expected: "LIKE YYYY-MM-DD HH:MM:SS",
        });
        return INVALID;
    }
    _parseNumber(context, argument, otherArguments) {
        const totalLength = otherArguments.length + 1;
        if (totalLength === 1) {
            return this._parseObject(context, DateTime.fromMillis(argument, {
                zone: "UTC",
            }));
        }
        if (totalLength !== 6) {
            this.setIssueForContext(context, totalLength > 6
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 6,
                    exact: true,
                    received: totalLength,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 6,
                    exact: true,
                    received: totalLength,
                });
            return INVALID;
        }
        for (const otherArgument of otherArguments) {
            const allowedTypes = [ParsedType.number], parsedType = getParsedType(otherArgument);
            if (!isOneOf(allowedTypes, parsedType)) {
                this.setIssueForContext(context, {
                    code: "invalid_type",
                    expected: allowedTypes,
                    received: parsedType,
                });
                return INVALID;
            }
        }
        const [year, month, day, hour, minute, second] = [argument, ...otherArguments];
        return this._parseString(context, `${pad(year, 4)}-${pad(month)}-${pad(day)} ${pad(hour)}:${pad(minute)}:${pad(second)}`);
    }
    _parseObject(context, argument) {
        // Should be [Timestamp]
        if (Timestamp.isTimestamp(argument))
            return OK(new TimestampClass(argument.year, argument.month, argument.day, argument.hour, argument.minute, argument.second));
        // Should be [globalThis.Date]
        const jsDate = isValidDate(argument);
        if (jsDate.isOfSameType) {
            if (jsDate.isValid) {
                const [year, month, day, hour, minute, second, millisecond] = [
                    jsDate.data.getFullYear(),
                    jsDate.data.getMonth() + 1,
                    jsDate.data.getDate(),
                    jsDate.data.getHours(),
                    jsDate.data.getMinutes(),
                    jsDate.data.getSeconds(),
                    jsDate.data.getMilliseconds(),
                ];
                return this._parseString(context, `${pad(year, 4)}-${pad(month)}-${pad(day)} ${pad(hour)}:${pad(minute)}:${pad(second)}${millisecond === 0 ? "" : `.${pad(millisecond, 3)}`}`);
            }
            this.setIssueForContext(context, jsDate.error);
            return INVALID;
        }
        // Should be [luxon.DateTime]
        const luxonDate = isValidDateTime(argument);
        if (luxonDate.isOfSameType) {
            if (luxonDate.isValid) {
                const [year, month, day, hour, minute, second, millisecond] = [
                    luxonDate.data.year,
                    luxonDate.data.month,
                    luxonDate.data.day,
                    luxonDate.data.hour,
                    luxonDate.data.minute,
                    luxonDate.data.second,
                    luxonDate.data.millisecond,
                ];
                return this._parseString(context, `${pad(year, 4)}-${pad(month)}-${pad(day)} ${pad(hour)}:${pad(minute)}:${pad(second)}${millisecond === 0 ? "" : `.${pad(millisecond, 3)}`}`);
            }
            this.setIssueForContext(context, luxonDate.error);
            return INVALID;
        }
        // Should be [TimeObject]
        const parsedObject = hasKeys(argument, [
            ["year", "number"],
            ["month", "number"],
            ["day", "number"],
            ["hour", "number"],
            ["minute", "number"],
            ["second", "number"],
        ]);
        if (!parsedObject.success) {
            switch (true) {
                case parsedObject.otherKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "unrecognized_keys",
                        keys: parsedObject.otherKeys,
                    });
                    break;
                case parsedObject.missingKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "missing_keys",
                        keys: parsedObject.missingKeys,
                    });
                    break;
                case parsedObject.invalidKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "invalid_key_type",
                        ...parsedObject.invalidKeys[0],
                    });
                    break;
            }
            return INVALID;
        }
        const { year, month, day, hour, minute, second } = parsedObject.obj;
        if (year % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
                received: year,
            });
        }
        if (month % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
                received: month,
            });
        }
        if (month < 1) {
            throwPgTPError({
                code: "too_small",
                minimum: 1,
                type: "number",
                inclusive: true,
                received: month,
            });
        }
        if (month > 12) {
            throwPgTPError({
                code: "too_big",
                maximum: 12,
                type: "number",
                inclusive: true,
                received: month,
            });
        }
        if (day % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
                received: day,
            });
        }
        if (day < 1) {
            throwPgTPError({
                code: "too_small",
                minimum: 1,
                type: "number",
                inclusive: true,
                received: day,
            });
        }
        if (day > 31) {
            throwPgTPError({
                code: "too_big",
                maximum: 31,
                type: "number",
                inclusive: true,
                received: day,
            });
        }
        if (hour % 1 !== 0) {
            this.setIssueForContext(context, {
                code: "not_whole",
                received: hour,
            });
            return INVALID;
        }
        if (hour < 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: hour,
            });
            return INVALID;
        }
        if (hour > 23) {
            this.setIssueForContext(context, {
                code: "too_big",
                maximum: 23,
                type: "number",
                inclusive: true,
                received: hour,
            });
            return INVALID;
        }
        if (minute % 1 !== 0) {
            this.setIssueForContext(context, {
                code: "not_whole",
                received: minute,
            });
            return INVALID;
        }
        if (minute < 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: minute,
            });
            return INVALID;
        }
        if (minute > 59) {
            this.setIssueForContext(context, {
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
                received: minute,
            });
            return INVALID;
        }
        if (second < 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: second,
            });
            return INVALID;
        }
        if (second >= 60) {
            this.setIssueForContext(context, {
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
                received: second,
            });
            return INVALID;
        }
        return OK(new TimestampClass(year, month, day, hour, minute, second));
    }
    isTimestamp(object) {
        return object instanceof TimestampClass;
    }
}
const Timestamp = new TimestampConstructorClass();
class TimestampClass extends PgTPBase {
    constructor(_year, _month, _day, _hour, _minute, _second) {
        super();
        this._year = _year;
        this._month = _month;
        this._day = _day;
        this._hour = _hour;
        this._minute = _minute;
        this._second = _second;
    }
    _equals(context) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = Timestamp.safeFrom(...context.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(context, parsed.error.issue);
        return INVALID;
    }
    toString(style = TimestampStyle.ISO) {
        const asTimestampTZ = TimestampTZ.from({
            year: this._year,
            month: this._month,
            day: this._day,
            hour: this._hour,
            minute: this._minute,
            second: this._second,
            offset: {
                direction: OffsetDirection.plus,
                hour: 0,
                minute: 0,
            },
        });
        return asTimestampTZ.toString(style);
    }
    toNumber() {
        return this.toDateTime("UTC").toMillis();
    }
    toJSON() {
        return {
            year: this._year,
            month: this._month,
            day: this._day,
            hour: this._hour,
            minute: this._minute,
            second: this._second,
        };
    }
    toDate() {
        return Date.from({
            year: this._year,
            month: this._month,
            day: this._day,
        });
    }
    toTime() {
        return Time.from({
            hour: this._hour,
            minute: this._minute,
            second: this._second,
        });
    }
    toDateTime(zone) {
        return DateTime.fromObject({
            year: this._year,
            month: this._month,
            day: this._day,
            hour: this._hour,
            minute: this._minute,
            second: Math.trunc(this._second),
            millisecond: Math.round((this._second % 1) * 1000),
        }, { zone });
    }
    toJSDate(zone) {
        return this.toDateTime(zone).toJSDate();
    }
    get year() {
        return this._year;
    }
    set year(year) {
        const parsedType = getParsedType(year);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (year % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
                received: year,
            });
        }
        this._year = year;
    }
    get month() {
        return this._month;
    }
    set month(month) {
        const parsedType = getParsedType(month);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (month % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
                received: month,
            });
        }
        if (month < 1) {
            throwPgTPError({
                code: "too_small",
                minimum: 1,
                type: "number",
                inclusive: true,
                received: month,
            });
        }
        if (month > 12) {
            throwPgTPError({
                code: "too_big",
                maximum: 12,
                type: "number",
                inclusive: true,
                received: month,
            });
        }
        this._month = month;
    }
    get day() {
        return this._day;
    }
    set day(day) {
        const parsedType = getParsedType(day);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (day % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
                received: day,
            });
        }
        if (day < 1) {
            throwPgTPError({
                code: "too_small",
                minimum: 1,
                type: "number",
                inclusive: true,
                received: day,
            });
        }
        if (day > 31) {
            throwPgTPError({
                code: "too_big",
                maximum: 31,
                type: "number",
                inclusive: true,
                received: day,
            });
        }
        this._day = day;
    }
    get hour() {
        return this._hour;
    }
    set hour(hour) {
        const parsedType = getParsedType(hour);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (hour % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
                received: hour,
            });
        }
        if (hour < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: hour,
            });
        }
        if (hour > 23) {
            throwPgTPError({
                code: "too_big",
                maximum: 23,
                type: "number",
                inclusive: true,
                received: hour,
            });
        }
        this._hour = hour;
    }
    get minute() {
        return this._minute;
    }
    set minute(minute) {
        const parsedType = getParsedType(minute);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (minute % 1 !== 0) {
            throwPgTPError({
                code: "not_whole",
                received: minute,
            });
        }
        if (minute < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: minute,
            });
        }
        if (minute > 59) {
            throwPgTPError({
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
                received: minute,
            });
        }
        this._minute = minute;
    }
    get second() {
        return this._second;
    }
    set second(second) {
        const parsedType = getParsedType(second);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        if (second < 0) {
            throwPgTPError({
                code: "too_small",
                minimum: 0,
                type: "number",
                inclusive: true,
                received: second,
            });
        }
        if (second >= 60) {
            throwPgTPError({
                code: "too_big",
                maximum: 59,
                type: "number",
                inclusive: true,
                received: second,
            });
        }
        this._second = second;
    }
    get value() {
        return this.toString();
    }
    set value(timestamp) {
        const parsed = Timestamp.safeFrom(timestamp);
        if (parsed.success) {
            this._year = parsed.data.year;
            this._month = parsed.data.month;
            this._day = parsed.data.day;
            this._hour = parsed.data.hour;
            this._minute = parsed.data.minute;
            this._second = parsed.data.second;
        }
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(timestamp) {
        const parsed = Timestamp.safeFrom(timestamp);
        if (parsed.success) {
            this._year = parsed.data.year;
            this._month = parsed.data.month;
            this._day = parsed.data.day;
            this._hour = parsed.data.hour;
            this._minute = parsed.data.minute;
            this._second = parsed.data.second;
        }
        else
            throw parsed.error;
    }
}
export { Timestamp };
//# sourceMappingURL=Timestamp.js.map