import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { throwPgTPError } from "../../util/throwPgTPError.js";
import { Character } from "../Character/Character.js";
import { CharacterVarying } from "../Character/CharacterVarying.js";
import { Name } from "../Character/Name.js";
import { Text } from "../Character/Text.js";
import { UUID } from "../UUID/UUID.js";
/* eslint-disable brace-style */
class EnumConstructorClass extends PgTPConstructorBase {
    constructor(_enums) {
        super();
        this._enums = _enums;
        /* eslint-enable brace-style */
        const allowedTypes = [ParsedType.array], parsedType = getParsedType(_enums);
        if (!isOneOf(allowedTypes, parsedType)) {
            throwPgTPError({
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
        }
        for (const enumValue of _enums) {
            if (typeof enumValue !== "string") {
                throwPgTPError({
                    code: "invalid_type",
                    expected: [ParsedType.string],
                    received: getParsedType(enumValue),
                });
            }
        }
        this._enums = [...new Set(_enums)];
    }
    _parse(context) {
        if (context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        const [argument] = context.data, allowedTypes = [ParsedType.string, ParsedType.object], parsedType = getParsedType(argument);
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case ParsedType.string:
                return this._parseString(context, argument);
            default:
                return this._parseObject(context, argument);
        }
    }
    _parseString(context, argument) {
        if (!this.enums.includes(argument)) {
            this.setIssueForContext(context, {
                code: "invalid_string",
                expected: this.enums,
                received: argument,
            });
            return INVALID;
        }
        return OK(new EnumClass(argument, this._enums));
    }
    _parseObject(context, argument) {
        if (this.isAnyEnum(argument))
            return this._parseString(context, argument.value);
        if (Character.isAnyCharacter(argument))
            return this._parseString(context, argument.value);
        if (CharacterVarying.isAnyCharacterVarying(argument))
            return this._parseString(context, argument.value);
        if (Name.isName(argument))
            return this._parseString(context, argument.value);
        if (Text.isText(argument))
            return this._parseString(context, argument.value);
        if (UUID.isUUID(argument))
            return this._parseString(context, argument.value);
        const parsedObject = hasKeys(argument, [["value", "string"]]);
        if (parsedObject.success)
            return this._parseString(context, parsedObject.obj.value);
        switch (true) {
            case parsedObject.otherKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "unrecognized_keys",
                    keys: parsedObject.otherKeys,
                });
                break;
            case parsedObject.missingKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "missing_keys",
                    keys: parsedObject.missingKeys,
                });
                break;
            case parsedObject.invalidKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "invalid_key_type",
                    ...parsedObject.invalidKeys[0],
                });
                break;
        }
        return INVALID;
    }
    isEnum(object, enums) {
        if (Array.isArray(enums))
            return this.setEnums(enums).isEnum(object);
        return (object instanceof EnumClass &&
            Array.isArray(object.enums) &&
            object.enums.length === this._enums.length &&
            object.enums.every(v => this._enums.includes(v)));
    }
    isAnyEnum(object) {
        return object instanceof EnumClass;
    }
    setEnums(enums) {
        return new EnumConstructorClass(enums);
    }
    get enums() {
        return this._enums;
    }
}
const Enum = new EnumConstructorClass(["No", "Enums", "Setupped", "Yet"]);
/* eslint-disable brace-style */
class EnumClass extends PgTPBase {
    constructor(_enum, _enums) {
        super();
        this._enum = _enum;
        this._enums = _enums;
    }
    /* eslint-enable brace-style */
    _equals(input) {
        const parsed = new EnumConstructorClass(this._enums).safeFrom(...input.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(input, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return this._enum;
    }
    toJSON() {
        return {
            value: this._enum,
        };
    }
    get enum() {
        return this._enum;
    }
    set enum(value) {
        const parsed = new EnumConstructorClass(this._enums).safeFrom(value);
        if (parsed.success)
            this._enum = parsed.data.toString();
        else
            throw parsed.error;
    }
    get value() {
        return this._enum;
    }
    set value(character) {
        const parsed = new EnumConstructorClass(this._enums).safeFrom(character);
        if (parsed.success)
            this._enum = parsed.data.toString();
        else
            throw parsed.error;
    }
    get postgres() {
        return this._enum;
    }
    set postgres(character) {
        const parsed = new EnumConstructorClass(this._enums).safeFrom(character);
        if (parsed.success)
            this._enum = parsed.data.toString();
        else
            throw parsed.error;
    }
    get enums() {
        return this._enums;
    }
}
export { Enum };
//# sourceMappingURL=Enum.js.map