import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { throwPgTPError } from "../../util/throwPgTPError.js";
import { Point } from "./Point.js";
var Connection;
(function (Connection) {
    Connection["open"] = "open";
    Connection["closed"] = "closed";
})(Connection || (Connection = {}));
// [] = open, () = closed
const connections = ["open", "closed"];
class PathConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        const [argument, ...otherArguments] = context.data, allowedTypes = [ParsedType.string, ParsedType.object, ParsedType.array], parsedType = getParsedType(argument);
        if (parsedType !== ParsedType.object && context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case "string":
                return this._parseString(context, argument);
            case "array":
                return this._parseArray(context, argument);
            default:
                return this._parseObject(context, argument, otherArguments);
        }
    }
    _parseString(context, argument) {
        // Remove all whitespace
        argument = argument.replaceAll(/\s/g, "");
        if (/^\(\((?:-?\d+(\.\d+)?|NaN),(?:-?\d+(\.\d+)?|NaN)\)(,\((?:-?\d+(\.\d+)?|NaN),(?:-?\d+(\.\d+)?|NaN)\))*\)$/.test(argument)) {
            const points = argument
                .slice(1, -1)
                .split("),(")
                .join("), (")
                .split(", ")
                .map(p => Point.safeFrom(p)), invalidPoint = points.find(p => !p.success);
            /* c8 ignore start */
            if (invalidPoint?.success === false) {
                this.setIssueForContext(context, invalidPoint.error.issue);
                return INVALID;
            }
            /* c8 ignore stop */
            return OK(new PathClass(
            //@ts-expect-error - They are all valid at this point
            points.map(range => range.data), Connection.closed));
        }
        else if (/^\[\((?:-?\d+(\.\d+)?|NaN),(?:-?\d+(\.\d+)?|NaN)\)(,\((?:-?\d+(\.\d+)?|NaN),(?:-?\d+(\.\d+)?|NaN)\))*]$/.test(argument)) {
            const points = argument
                .slice(1, -1)
                .split("),(")
                .join("), (")
                .split(", ")
                .map(point => Point.safeFrom(point)), invalidPoint = points.find(point => !point.success);
            /* c8 ignore start */
            if (invalidPoint?.success === false) {
                this.setIssueForContext(context, invalidPoint.error.issue);
                return INVALID;
            }
            /* c8 ignore stop */
            return OK(new PathClass(
            //@ts-expect-error - They are all valid at this point
            points.map(range => range.data), Connection.open));
        }
        this.setIssueForContext(context, {
            code: "invalid_string",
            received: argument,
            expected: "LIKE ((x,y),...) || [(x,y),...]",
        });
        return INVALID;
    }
    _parseArray(context, argument) {
        if (argument.length === 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                type: "array",
                minimum: 1,
                inclusive: true,
                received: argument.length,
            });
            return INVALID;
        }
        const points = argument.map(point => Point.safeFrom(point)), invalidPoint = points.find(point => !point.success);
        if (invalidPoint?.success === false) {
            this.setIssueForContext(context, invalidPoint.error.issue);
            return INVALID;
        }
        return OK(new PathClass(
        //@ts-expect-error - They are all valid at this point
        points.map(range => range.data), Connection.closed));
    }
    _parseObject(context, argument, otherArguments) {
        // Input should be [Path]
        if (Path.isPath(argument)) {
            if (otherArguments.length > 0) {
                this.setIssueForContext(context, {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: otherArguments.length,
                });
                return INVALID;
            }
            return OK(new PathClass(argument.points, argument.connection));
        }
        // Input should be [Point, ...]
        if (Point.isPoint(argument)) {
            const points = otherArguments.map(point => Point.safeFrom(point)), invalidPoint = points.find(point => !point.success);
            if (invalidPoint?.success === false) {
                this.setIssueForContext(context, invalidPoint.error.issue);
                return INVALID;
            }
            return OK(new PathClass(
            //@ts-expect-error - They are all valid at this point
            [argument, ...points.map(range => range.data)], Connection.closed));
        }
        // Input should be [PathObject | RawPathObject]
        if (otherArguments.length > 0) {
            this.setIssueForContext(context, {
                code: "too_big",
                type: "arguments",
                maximum: 1,
                exact: true,
                received: otherArguments.length,
            });
            return INVALID;
        }
        const parsedObject = hasKeys(argument, [
            ["points", "array"],
            ["connection", "string"],
        ]);
        if (!parsedObject.success) {
            switch (true) {
                case parsedObject.otherKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "unrecognized_keys",
                        keys: parsedObject.otherKeys,
                    });
                    break;
                case parsedObject.missingKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "missing_keys",
                        keys: parsedObject.missingKeys,
                    });
                    break;
                case parsedObject.invalidKeys.length > 0:
                    this.setIssueForContext(context, {
                        code: "invalid_key_type",
                        ...parsedObject.invalidKeys[0],
                    });
                    break;
            }
            return INVALID;
        }
        const { points, connection } = parsedObject.obj, parsedPoints = points.map(point => Point.safeFrom(point)), invalidPoint = parsedPoints.find(point => !point.success);
        if (invalidPoint?.success === false) {
            this.setIssueForContext(context, invalidPoint.error.issue);
            return INVALID;
        }
        if (parsedPoints.length === 0) {
            this.setIssueForContext(context, {
                code: "too_small",
                type: "array",
                minimum: 1,
                inclusive: true,
                received: parsedPoints.length,
            });
            return INVALID;
        }
        if (!connections.includes(connection)) {
            this.setIssueForContext(context, {
                code: "invalid_string",
                expected: connections,
                received: connection,
            });
            return INVALID;
        }
        return OK(new PathClass(
        //@ts-expect-error - They are all valid at this point
        parsedPoints.map(range => range.data), connection));
    }
    isPath(object) {
        return object instanceof PathClass;
    }
}
const Path = new PathConstructorClass();
class PathClass extends PgTPBase {
    constructor(_points, _connection) {
        super();
        this._points = _points;
        this._connection = _connection;
    }
    _equals(context) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = Path.safeFrom(...context.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(context, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return this._connection === Connection.closed ? `(${this._points.map(p => p.toString()).join(",")})` : `[${this._points.map(p => p.toString()).join(",")}]`;
    }
    toJSON() {
        return {
            points: this._points.map(p => p.toJSON()),
            connection: this._connection,
        };
    }
    get points() {
        return this._points;
    }
    set points(points) {
        const parsedType = getParsedType(points);
        if (parsedType !== "array") {
            throwPgTPError({
                code: "invalid_type",
                expected: "array",
                received: parsedType,
            });
        }
        if (points.length === 0) {
            throwPgTPError({
                code: "too_small",
                type: "array",
                minimum: 1,
                inclusive: true,
                received: points.length,
            });
        }
        const finalPoints = points.map(point => Point.safeFrom(point)), invalidPoint = finalPoints.find(point => !point.success);
        if (invalidPoint?.success === false)
            throwPgTPError(invalidPoint.error.issue);
        //@ts-expect-error - They are all valid at this point
        this._points = finalPoints.map(point => point.data);
    }
    get connection() {
        return this._connection;
    }
    set connection(connection) {
        const parsedType = getParsedType(connection);
        if (parsedType !== "string") {
            throwPgTPError({
                code: "invalid_type",
                expected: "string",
                received: parsedType,
            });
        }
        if (!connections.includes(connection)) {
            throwPgTPError({
                code: "invalid_string",
                expected: connections,
                received: connection,
            });
        }
        this._connection = connection;
    }
    get value() {
        return this.toString();
    }
    set value(path) {
        const parsed = Path.safeFrom(path);
        if (parsed.success) {
            this._connection = parsed.data.connection;
            this._points = parsed.data.points;
        }
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(path) {
        const parsed = Path.safeFrom(path);
        if (parsed.success) {
            this._connection = parsed.data.connection;
            this._points = parsed.data.points;
        }
        else
            throw parsed.error;
    }
}
export { Connection, Path };
//# sourceMappingURL=Path.js.map