/* eslint-disable unicorn/filename-case */
import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
class JSONConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        if (context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        const [argument] = context.data, allowedTypes = [ParsedType.number, ParsedType.string, ParsedType.object, ParsedType.null, ParsedType.array, ParsedType.boolean], parsedType = getParsedType(argument);
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case ParsedType.object:
                return this._parseObject(context, argument);
            case ParsedType.array:
            case ParsedType.number:
            case ParsedType.boolean:
            case ParsedType.null:
                return this._tryParse(context, argument);
            default:
                return this._parseString(context, argument);
        }
    }
    _parseString(context, argument) {
        try {
            const parsed = globalThis.JSON.parse(argument);
            return OK(new JSONClass(parsed));
        }
        catch {
            this.setIssueForContext(context, {
                code: "invalid_json",
                received: argument,
            });
            return INVALID;
        }
    }
    _tryParse(context, argument) {
        try {
            const parsed = globalThis.JSON.parse(globalThis.JSON.stringify(argument));
            return OK(new JSONClass(parsed));
            /* c8 ignore next 7 */
        }
        catch {
            this.setIssueForContext(context, {
                code: "invalid_json",
                received: `${argument}`,
            });
            return INVALID;
        }
    }
    _parseObject(context, argument) {
        if (this.isJSON(argument))
            return OK(new JSONClass(argument.json));
        const parsedObject = hasKeys(argument, [["value", "string"]]);
        if (parsedObject.success)
            return this._parseString(context, parsedObject.obj.value);
        return OK(new JSONClass(argument));
    }
    isJSON(object) {
        return object instanceof JSONClass;
    }
}
const JSON = new JSONConstructorClass();
class JSONClass extends PgTPBase {
    constructor(_json) {
        super();
        this._json = _json;
    }
    _equals(input) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = JSON.safeFrom(...input.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(input, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return globalThis.JSON.stringify(this._json);
    }
    toJSON() {
        return {
            value: this.toString(),
        };
    }
    get json() {
        return this._json;
    }
    set json(json) {
        const parsed = JSON.safeFrom(json);
        if (parsed.success)
            this._json = parsed.data.json;
        else
            throw parsed.error;
    }
    get value() {
        return this.toString();
    }
    set value(json) {
        const parsed = JSON.safeFrom(json);
        if (parsed.success)
            this._json = parsed.data.json;
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(json) {
        const parsed = JSON.safeFrom(json);
        if (parsed.success)
            this._json = parsed.data.json;
        else
            throw parsed.error;
    }
}
export { JSON };
//# sourceMappingURL=JSON.js.map