/* eslint-disable unicorn/filename-case */
import { parseInteger } from "jsprim";
const MACAddress = {
    from(argument) {
        if (typeof argument === "string" || typeof argument === "number")
            return new MACAddressClass(argument);
        else if (MACAddress.isMACAddress(argument))
            return new MACAddressClass(argument.toJSON().MACAddress);
        else
            return new MACAddressClass(argument.MACAddress);
    },
    isMACAddress(object) {
        return object instanceof MACAddressClass;
    },
};
class MACAddressClass {
    constructor(data) {
        switch (typeof data) {
            case "string":
                this._MACAddress = this._parseString(data);
                break;
            case "number":
                this._MACAddress = this._parseLong(data);
                break;
            default:
                throw new Error("Invalid MACAddress");
        }
    }
    _parseLong(input) {
        if (input !== Math.floor(input))
            throw new Error("Invalid MACAddress");
        if (input < 0 || input > 281474976710655)
            throw new Error("MACAddress must be 48-bit");
        return input;
    }
    _isDigit(c) {
        return /^[\da-f]$/.test(c);
    }
    _parseString(input) {
        input = input.toLowerCase();
        let position = 0, value = 0, octet = "", seperator = null, chr;
        function isSeparator(separator) {
            if (seperator !== null)
                return separator === seperator;
            if (separator !== ":" && separator !== "-" && separator !== ".")
                return false;
            seperator = separator;
            return true;
        }
        function process() {
            if (octet.length === 0)
                throw new Error(`Invalid MACAddress, expected to find a hexadecimal number before ${JSON.stringify(seperator)}`);
            else if (octet.length > 2)
                throw new Error(`Invalid MACAddress, too many hexadecimal digits in ${JSON.stringify(octet)}`);
            else if (position < 6) {
                const temporary = parseInteger(octet, { base: 16 });
                if (temporary instanceof Error)
                    throw new Error(`Invalid MACAddress, "${octet}" is not a valid hexadecimal number`);
                value *= 256;
                value += temporary;
                position += 1;
                octet = "";
            }
            else
                throw new Error("Invalid MACAddress, too many octets");
        }
        for (const element of input) {
            chr = element;
            if (isSeparator(chr))
                process();
            else if (this._isDigit(chr))
                octet += chr;
            else
                throw new Error(`Invalid MACAddress, unrecognized character ${JSON.stringify(chr)}`);
        }
        if (isSeparator(chr))
            throw new Error(`Invalid MACAddress, trailing ${JSON.stringify(seperator)}`);
        if (position === 0) {
            if (octet.length < 12)
                throw new Error("Invalid MACAddress, too few octets");
            else if (octet.length > 12)
                throw new Error("Invalid MACAddress, too many octets");
            const temporary = parseInteger(octet, { base: 16 });
            if (temporary instanceof Error)
                throw new Error(`Invalid MACAddress, "${octet}" is not a valid hexadecimal number`);
            value = temporary;
        }
        else {
            process();
            if (position < 6)
                throw new Error("Invalid MACAddress, too few octets");
            else if (position > 6)
                throw new Error("Invalid MACAddress, too many octets");
        }
        return value;
    }
    toString() {
        let result = "";
        const fields = [
            /*
             * JavaScript converts numbers to 32-bit integers when doing bitwise
             * arithmetic, so we have to handle the first two parts of the number
             * differently.
             */
            (this._MACAddress / 1099511627776) & 0xff,
            (this._MACAddress / 4294967296) & 0xff,
            (this._MACAddress >>> 24) & 0xff,
            (this._MACAddress >>> 16) & 0xff,
            (this._MACAddress >>> 8) & 0xff,
            this._MACAddress & 0xff,
        ];
        for (const [index, field] of fields.entries()) {
            if (index !== 0)
                result += ":";
            const octet = field.toString(16);
            if (octet.length === 1)
                result += "0";
            result += octet;
        }
        return result;
    }
    toJSON() {
        return {
            MACAddress: this.toString(),
        };
    }
    toLong() {
        return this._MACAddress;
    }
    equals(otherMACAddress) {
        if (typeof otherMACAddress === "string")
            return otherMACAddress.toLowerCase() === this.toString().toLowerCase();
        else if (typeof otherMACAddress === "number")
            return otherMACAddress === this.toLong();
        else if (MACAddress.isMACAddress(otherMACAddress))
            return otherMACAddress.toString().toLowerCase() === this.toString().toLowerCase();
        else
            return otherMACAddress.MACAddress.toLowerCase() === this.toString().toLowerCase();
    }
    get MACAddress() {
        return this.toString();
    }
    set MACAddress(data) {
        switch (typeof data) {
            case "string":
                this._MACAddress = this._parseString(data);
                break;
            case "number":
                this._MACAddress = this._parseLong(data);
                break;
            default:
                throw new Error("Invalid MACAddress");
        }
    }
}
export { MACAddress };
//# sourceMappingURL=MACAddress.js.map