import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
class Int4ConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        if (context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        const [argument] = context.data, allowedTypes = [ParsedType.number, ParsedType.string, ParsedType.object], parsedType = getParsedType(argument);
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case ParsedType.number:
                return this._parseNumber(context, argument);
            case ParsedType.string:
                return this._parseString(context, argument);
            default:
                return this._parseObject(context, argument);
        }
    }
    _parseNumber(context, argument) {
        if (Number.isNaN(argument)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: "number",
                received: "nan",
            });
            return INVALID;
        }
        if (!Number.isFinite(argument)) {
            this.setIssueForContext(context, {
                code: "not_finite",
                received: argument,
            });
            return INVALID;
        }
        if (argument % 1 !== 0) {
            this.setIssueForContext(context, {
                code: "not_whole",
                received: argument,
            });
            return INVALID;
        }
        if (argument < -2147483648) {
            this.setIssueForContext(context, {
                code: "too_small",
                type: "number",
                minimum: -2147483648,
                inclusive: true,
                received: argument,
            });
            return INVALID;
        }
        if (argument > 2147483647) {
            this.setIssueForContext(context, {
                code: "too_big",
                type: "number",
                maximum: 2147483647,
                inclusive: true,
                received: argument,
            });
            return INVALID;
        }
        return OK(new Int4Class(argument));
    }
    _parseString(context, argument) {
        const parsed = Number.parseFloat(argument);
        return this._parseNumber(context, parsed);
    }
    _parseObject(context, argument) {
        if (this.isInt4(argument))
            return OK(new Int4Class(argument.value));
        const parsedObject = hasKeys(argument, [["value", "number"]]);
        if (parsedObject.success)
            return this._parseNumber(context, parsedObject.obj.value);
        switch (true) {
            case parsedObject.otherKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "unrecognized_keys",
                    keys: parsedObject.otherKeys,
                });
                break;
            case parsedObject.missingKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "missing_keys",
                    keys: parsedObject.missingKeys,
                });
                break;
            case parsedObject.invalidKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "invalid_key_type",
                    ...parsedObject.invalidKeys[0],
                });
                break;
        }
        return INVALID;
    }
    isInt4(object) {
        return object instanceof Int4Class;
    }
}
const Int4 = new Int4ConstructorClass();
class Int4Class extends PgTPBase {
    constructor(_int4) {
        super();
        this._int4 = _int4;
    }
    _equals(input) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = Int4.safeFrom(...input.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toNumber() === this.toNumber(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(input, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return this._int4.toString();
    }
    toNumber() {
        return this._int4;
    }
    toJSON() {
        return {
            value: this._int4,
        };
    }
    get int4() {
        return this._int4;
    }
    set int4(int4) {
        const parsed = Int4.safeFrom(int4);
        if (parsed.success)
            this._int4 = parsed.data.toNumber();
        else
            throw parsed.error;
    }
    get value() {
        return this._int4;
    }
    set value(int4) {
        const parsed = Int4.safeFrom(int4);
        if (parsed.success)
            this._int4 = parsed.data.toNumber();
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(int4) {
        const parsed = Int4.safeFrom(int4);
        if (parsed.success)
            this._int4 = parsed.data.toNumber();
        else
            throw parsed.error;
    }
}
export { Int4 };
//# sourceMappingURL=Int4.js.map