import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
class Int8ConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        if (context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        const [argument] = context.data, allowedTypes = [ParsedType.bigint, ParsedType.number, ParsedType.string, ParsedType.object], parsedType = getParsedType(argument);
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case ParsedType.bigint:
                return this._parseBigint(context, argument);
            case ParsedType.number:
                return this._parseNumber(context, argument);
            case ParsedType.string:
                return this._parseString(context, argument);
            default:
                return this._parseObject(context, argument);
        }
    }
    _parseNumber(context, argument) {
        if (argument % 1 !== 0) {
            this.setIssueForContext(context, {
                code: "not_whole",
                received: argument,
            });
            return INVALID;
        }
        return this._parseBigint(context, BigInt(argument));
    }
    _parseBigint(context, argument) {
        if (argument < BigInt("-9223372036854775808")) {
            this.setIssueForContext(context, {
                code: "too_small",
                type: "bigint",
                minimum: BigInt("-9223372036854775808"),
                inclusive: true,
                received: argument,
            });
            return INVALID;
        }
        if (argument > BigInt("9223372036854775807")) {
            this.setIssueForContext(context, {
                code: "too_big",
                type: "bigint",
                maximum: BigInt("9223372036854775807"),
                inclusive: true,
                received: argument,
            });
            return INVALID;
        }
        return OK(new Int8Class(argument));
    }
    _parseString(context, argument) {
        try {
            return this._parseBigint(context, BigInt(argument));
        }
        catch {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: "bigint",
                received: "string",
            });
            return INVALID;
        }
    }
    _parseObject(context, argument) {
        if (this.isInt8(argument))
            return OK(new Int8Class(argument.int8));
        const parsedObject = hasKeys(argument, [["value", "string"]]);
        if (parsedObject.success)
            return this._parseString(context, parsedObject.obj.value);
        switch (true) {
            case parsedObject.otherKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "unrecognized_keys",
                    keys: parsedObject.otherKeys,
                });
                break;
            case parsedObject.missingKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "missing_keys",
                    keys: parsedObject.missingKeys,
                });
                break;
            case parsedObject.invalidKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "invalid_key_type",
                    ...parsedObject.invalidKeys[0],
                });
                break;
        }
        return INVALID;
    }
    isInt8(object) {
        return object instanceof Int8Class;
    }
}
const Int8 = new Int8ConstructorClass();
class Int8Class extends PgTPBase {
    constructor(_int8) {
        super();
        this._int8 = _int8;
    }
    _equals(input) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = Int8.safeFrom(...input.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toBigint() === this.toBigint(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(input, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return this._int8.toString();
    }
    toNumber() {
        return Number(this._int8);
    }
    toBigint() {
        return this._int8;
    }
    toJSON() {
        return {
            value: this.toString(),
        };
    }
    get int8() {
        return this._int8;
    }
    set int8(int8) {
        const parsed = Int8.safeFrom(int8);
        if (parsed.success)
            this._int8 = parsed.data.toBigint();
        else
            throw parsed.error;
    }
    get value() {
        return this.toNumber();
    }
    set value(int8) {
        const parsed = Int8.safeFrom(int8);
        if (parsed.success)
            this._int8 = parsed.data.int8;
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(int8) {
        const parsed = Int8.safeFrom(int8);
        if (parsed.success)
            this._int8 = parsed.data.int8;
        else
            throw parsed.error;
    }
}
export { Int8 };
//# sourceMappingURL=Int8.js.map