/* eslint-disable unicorn/filename-case */
import { randomUUID } from "node:crypto";
import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { Character } from "../Character/Character.js";
import { CharacterVarying } from "../Character/CharacterVarying.js";
import { Name } from "../Character/Name.js";
import { Text } from "../Character/Text.js";
class UUIDConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        if (context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                }
                : {
                    code: "too_small",
                    type: "arguments",
                    minimum: 1,
                    exact: true,
                    received: context.data.length,
                });
            return INVALID;
        }
        const [argument] = context.data, allowedTypes = [ParsedType.string, ParsedType.object], parsedType = getParsedType(argument);
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case ParsedType.string:
                return this._parseString(context, argument);
            default:
                return this._parseObject(context, argument);
        }
    }
    _parseString(context, argument) {
        if (/^([\da-f]{8}(?:-[\da-f]{4}){3}-[\da-f]{12})$/i.test(argument))
            return OK(new UUIDClass(argument));
        if (/^{([\da-f]{8}(?:-[\da-f]{4}){3}-[\da-f]{12})}$/i.test(argument))
            return OK(new UUIDClass(argument.slice(1, -1)));
        if (/^([\da-f]{32})$/i.test(argument))
            return OK(new UUIDClass(argument.replace(/(.{8})(.{4})(.{4})(.{4})(.{12})/, "$1-$2-$3-$4-$5")));
        this.setIssueForContext(context, {
            code: "invalid_string",
            received: argument,
            expected: "LIKE xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
        });
        return INVALID;
    }
    _parseObject(context, argument) {
        if (this.isUUID(argument))
            return OK(new UUIDClass(argument.value));
        if (Character.isAnyCharacter(argument))
            return this._parseString(context, argument.value);
        if (CharacterVarying.isAnyCharacterVarying(argument))
            return this._parseString(context, argument.value);
        if (Name.isName(argument))
            return this._parseString(context, argument.value);
        if (Text.isText(argument))
            return this._parseString(context, argument.value);
        const parsedObject = hasKeys(argument, [["value", "string"]]);
        if (parsedObject.success)
            return this._parseString(context, parsedObject.obj.value);
        switch (true) {
            case parsedObject.otherKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "unrecognized_keys",
                    keys: parsedObject.otherKeys,
                });
                break;
            case parsedObject.missingKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "missing_keys",
                    keys: parsedObject.missingKeys,
                });
                break;
            case parsedObject.invalidKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "invalid_key_type",
                    ...parsedObject.invalidKeys[0],
                });
                break;
        }
        return INVALID;
    }
    generate(options) {
        return new UUIDClass(randomUUID(options));
    }
    isUUID(object) {
        return object instanceof UUIDClass;
    }
}
const UUID = new UUIDConstructorClass();
class UUIDClass extends PgTPBase {
    constructor(_uuid) {
        super();
        this._uuid = _uuid;
        this._uuid = _uuid.toLowerCase();
    }
    _equals(context) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = UUID.safeFrom(...context.data);
        if (parsed.success) {
            return OK({
                equals: parsed.data.toString() === this.toString(),
                data: parsed.data,
            });
        }
        this.setIssueForContext(context, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return this._uuid;
    }
    toJSON() {
        return {
            value: this._uuid,
        };
    }
    get uuid() {
        return this._uuid;
    }
    set uuid(uuid) {
        const parsed = UUID.safeFrom(uuid);
        if (parsed.success)
            this._uuid = parsed.data.toString();
        else
            throw parsed.error;
    }
    get value() {
        return this._uuid;
    }
    set value(uuid) {
        const parsed = UUID.safeFrom(uuid);
        if (parsed.success)
            this._uuid = parsed.data.toString();
        else
            throw parsed.error;
    }
    get postgres() {
        return this._uuid;
    }
    set postgres(uuid) {
        const parsed = UUID.safeFrom(uuid);
        if (parsed.success)
            this._uuid = parsed.data.toString();
        else
            throw parsed.error;
    }
}
export { UUID };
//# sourceMappingURL=UUID.js.map