import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { greaterThan } from "./greaterThan.js";
import { greaterThanOrEqual } from "./greaterThanOrEqual.js";
import { lessThan } from "./lessThan.js";
import { lessThanOrEqual } from "./lessThanOrEqual.js";
import { PgTPConstructorBase } from "./PgTPConstructorBase.js";
import { PgTPRangeBase } from "./PgTPRangeBase.js";
import { throwPgTPError } from "./throwPgTPError.js";
var LowerRange;
(function (LowerRange) {
    LowerRange["include"] = "[";
    LowerRange["exclude"] = "(";
})(LowerRange || (LowerRange = {}));
const lowerRange = ["[", "("];
var UpperRange;
(function (UpperRange) {
    UpperRange["include"] = "]";
    UpperRange["exclude"] = ")";
})(UpperRange || (UpperRange = {}));
const upperRange = ["]", ")"];
const getRange = (object, isObjectFunction, identifier) => {
    const Object = object;
    class RangeConstructorClass extends PgTPConstructorBase {
        constructor() {
            super();
        }
        _parse(context) {
            const [argument, secondArgument] = context.data, allowedTypes = [ParsedType.string, ParsedType.object, ParsedType.array], parsedType = getParsedType(argument);
            if (parsedType !== ParsedType.object && context.data.length !== 1) {
                this.setIssueForContext(context, context.data.length > 1
                    ? {
                        code: "too_big",
                        type: "arguments",
                        maximum: 1,
                        exact: true,
                        received: context.data.length,
                    }
                    : {
                        code: "too_small",
                        type: "arguments",
                        minimum: 1,
                        exact: true,
                        received: context.data.length,
                    });
                return INVALID;
            }
            if (!isOneOf(allowedTypes, parsedType)) {
                this.setIssueForContext(context, {
                    code: "invalid_type",
                    expected: allowedTypes,
                    received: parsedType,
                });
                return INVALID;
            }
            switch (parsedType) {
                case "string":
                    return this._parseString(context, argument);
                case "array":
                    return this._parseArray(context, argument);
                default:
                    return this._parseObject(context, argument, secondArgument);
            }
        }
        _parseString(context, argument) {
            if (argument === "empty") {
                return OK(new RangeClass({
                    lower: LowerRange.include,
                    upper: UpperRange.include,
                    values: null,
                }));
            }
            // If the 0 index is set then that automatically means -1 is set
            const [lower, upper] = [argument.at(0), argument.at(-1)];
            if (!lower || !lowerRange.includes(lower)) {
                this.setIssueForContext(context, {
                    code: "invalid_string",
                    expected: lowerRange,
                    received: lower || "",
                });
                return INVALID;
            }
            if (!upperRange.includes(upper)) {
                this.setIssueForContext(context, {
                    code: "invalid_string",
                    expected: upperRange,
                    received: upper,
                });
                return INVALID;
            }
            const value = argument
                .slice(1, -1)
                .split(",")
                .map(v => v.replaceAll('"', ""))
                .map(v => v.replaceAll("\\", ""));
            if (value.length !== 2) {
                this.setIssueForContext(context, value.length > 2
                    ? {
                        code: "too_big",
                        type: "array",
                        maximum: 2,
                        exact: true,
                        received: value.length,
                    }
                    : {
                        code: "too_small",
                        type: "array",
                        minimum: 2,
                        exact: true,
                        received: value.length,
                    });
                return INVALID;
            }
            const [lowerValue, upperValue] = value.map(v => Object.safeFrom(v));
            if (!lowerValue.success) {
                this.setIssueForContext(context, lowerValue.error.issue);
                return INVALID;
            }
            if (!upperValue.success) {
                this.setIssueForContext(context, upperValue.error.issue);
                return INVALID;
            }
            if (greaterThan(lowerValue.data, upperValue.data)) {
                this.setIssueForContext(context, {
                    code: "invalid_range_bound",
                    lower: lowerValue.data.toString(),
                    upper: upperValue.data.toString(),
                });
                return INVALID;
            }
            return OK(new RangeClass({
                lower: lower,
                upper: upper,
                values: [lowerValue.data, upperValue.data],
            }));
        }
        _parseArray(context, argument, options) {
            if (argument.length !== 2) {
                this.setIssueForContext(context, argument.length > 2
                    ? {
                        code: "too_big",
                        type: "array",
                        maximum: 2,
                        exact: true,
                        received: argument.length,
                    }
                    : {
                        code: "too_small",
                        type: "array",
                        minimum: 2,
                        exact: true,
                        received: argument.length,
                    });
                return INVALID;
            }
            const [lower, upper] = argument, lowerValue = Object.safeFrom(lower), upperValue = Object.safeFrom(upper);
            if (!lowerValue.success) {
                this.setIssueForContext(context, lowerValue.error.issue);
                return INVALID;
            }
            if (!upperValue.success) {
                this.setIssueForContext(context, upperValue.error.issue);
                return INVALID;
            }
            if (greaterThan(lowerValue.data, upperValue.data)) {
                this.setIssueForContext(context, {
                    code: "invalid_range_bound",
                    lower: lowerValue.data.toString(),
                    upper: upperValue.data.toString(),
                });
                return INVALID;
            }
            return OK(new RangeClass({
                lower: options?.lower ?? LowerRange.include,
                upper: options?.upper ?? UpperRange.exclude,
                values: [lowerValue.data, upperValue.data],
            }));
        }
        _parseObject(context, argument, secondArgument) {
            const parsedType = getParsedType(secondArgument);
            if (context.data.length > 2) {
                this.setIssueForContext(context, {
                    code: "too_big",
                    type: "arguments",
                    maximum: 2,
                    inclusive: true,
                    received: context.data.length,
                });
                return INVALID;
            }
            //Input should be [Range<DataType, DataTypeObject>]
            if (Range.isRange(argument)) {
                if (parsedType !== "undefined") {
                    this.setIssueForContext(context, {
                        code: "too_big",
                        type: "arguments",
                        maximum: 1,
                        exact: true,
                        received: context.data.length,
                    });
                    return INVALID;
                }
                return OK(new RangeClass({
                    lower: argument.lower,
                    upper: argument.upper,
                    values: argument.values,
                }));
            }
            //Input should be [DataType, DataType]
            if (isObjectFunction(argument)) {
                if (parsedType === "undefined") {
                    this.setIssueForContext(context, {
                        code: "too_small",
                        type: "arguments",
                        minimum: 2,
                        exact: true,
                        received: 1,
                    });
                    return INVALID;
                }
                const parsedObject = Object.safeFrom(secondArgument);
                if (!parsedObject.success) {
                    this.setIssueForContext(context, parsedObject.error.issue);
                    return INVALID;
                }
                if (greaterThan(argument, parsedObject.data)) {
                    this.setIssueForContext(context, {
                        code: "invalid_range_bound",
                        lower: argument.toString(),
                        upper: parsedObject.data.toString(),
                    });
                    return INVALID;
                }
                return OK(new RangeClass({
                    lower: LowerRange.include,
                    upper: UpperRange.exclude,
                    values: [argument, parsedObject.data],
                }));
            }
            //Input should be [RangeObject<DataType> | RawRangeObject<DataTypeObject>]
            if (parsedType !== "undefined") {
                this.setIssueForContext(context, {
                    code: "too_big",
                    type: "arguments",
                    maximum: 1,
                    exact: true,
                    received: context.data.length,
                });
                return INVALID;
            }
            const parsedObject = hasKeys(argument, [
                ["lower", "string"],
                ["upper", "string"],
                ["values", ["array", "null"]],
            ]);
            if (!parsedObject.success) {
                switch (true) {
                    case parsedObject.otherKeys.length > 0:
                        this.setIssueForContext(context, {
                            code: "unrecognized_keys",
                            keys: parsedObject.otherKeys,
                        });
                        break;
                    case parsedObject.missingKeys.length > 0:
                        this.setIssueForContext(context, {
                            code: "missing_keys",
                            keys: parsedObject.missingKeys,
                        });
                        break;
                    case parsedObject.invalidKeys.length > 0:
                        this.setIssueForContext(context, {
                            code: "invalid_key_type",
                            ...parsedObject.invalidKeys[0],
                        });
                        break;
                }
                return INVALID;
            }
            const { lower, upper, values: value } = parsedObject.obj;
            if (!lowerRange.includes(lower)) {
                this.setIssueForContext(context, {
                    code: "invalid_string",
                    expected: lowerRange,
                    received: lower,
                });
                return INVALID;
            }
            if (!upperRange.includes(upper)) {
                this.setIssueForContext(context, {
                    code: "invalid_string",
                    expected: upperRange,
                    received: upper,
                });
                return INVALID;
            }
            if (value === null) {
                return OK(new RangeClass({
                    lower,
                    upper,
                    values: value,
                }));
            }
            return this._parseArray(context, value, {
                lower,
                upper,
            });
        }
        isRange(object) {
            //@ts-expect-error - This is a hack to get around the fact that the value is private
            return object instanceof RangeClass && object._identifier === identifier;
        }
    }
    const Range = new RangeConstructorClass();
    class RangeClass extends PgTPRangeBase {
        constructor(data) {
            super();
            this._identifier = identifier;
            this._lower = data.lower;
            this._upper = data.upper;
            this._values = data.values === null ? null : data.values;
            this._checkEmpty();
        }
        _checkEmpty() {
            if (this._values &&
                ((this._lower === LowerRange.include && this._upper === UpperRange.exclude) ||
                    (this._lower === LowerRange.exclude && this._upper === UpperRange.include)) &&
                this._values[0].equals(this._values[1]))
                this._values = null;
        }
        _equals(input) {
            //@ts-expect-error - _equals receives the same context as _parse
            const parsed = Range.safeFrom(...input.data);
            if (parsed.success) {
                return OK({
                    equals: parsed.data.toString() === this.toString(),
                    data: parsed.data,
                });
            }
            this.setIssueForContext(input, parsed.error.issue);
            return INVALID;
        }
        _isWithinRange(input) {
            //@ts-expect-error - _equals receives the same context as _parse
            const parsed = Object.safeFrom(...input.data);
            if (parsed.success) {
                if (this._values === null) {
                    return OK({
                        isWithinRange: false,
                    });
                }
                return OK({
                    isWithinRange: (this._lower === LowerRange.include ? greaterThanOrEqual(parsed.data, this._values[0]) : greaterThan(parsed.data, this._values[0])) &&
                        (this._upper === UpperRange.include ? lessThanOrEqual(parsed.data, this._values[1]) : lessThan(parsed.data, this._values[1])),
                    data: parsed.data,
                });
            }
            this.setIssueForContext(input, parsed.error.issue);
            return INVALID;
        }
        toString() {
            if (this._values === null)
                return "empty";
            return `${this._lower}${this._values[0].toString()},${this._values[1].toString()}${this._upper}`;
        }
        toJSON() {
            return {
                lower: this._lower,
                upper: this._upper,
                values: this._values?.map(v => v.toJSON()) ?? null,
            };
        }
        get lower() {
            return this._lower;
        }
        set lower(value) {
            if (!lowerRange.includes(value)) {
                throwPgTPError({
                    code: "invalid_string",
                    expected: lowerRange,
                    received: value,
                });
            }
            this._lower = value;
            this._checkEmpty();
        }
        get upper() {
            return this._upper;
        }
        set upper(value) {
            if (!upperRange.includes(value)) {
                throwPgTPError({
                    code: "invalid_string",
                    expected: upperRange,
                    received: value,
                });
            }
            this._upper = value;
            this._checkEmpty();
        }
        get values() {
            return this._values;
        }
        set values(value) {
            if (value === null) {
                this._values = null;
                return;
            }
            const parsedType = getParsedType(value);
            if (parsedType !== "array") {
                throwPgTPError({
                    code: "invalid_type",
                    expected: "array",
                    received: parsedType,
                });
            }
            if (value.length !== 2) {
                throwPgTPError(value.length > 2
                    ? {
                        code: "too_big",
                        type: "array",
                        maximum: 2,
                        exact: true,
                        received: value.length,
                    }
                    : {
                        code: "too_small",
                        type: "array",
                        minimum: 2,
                        exact: true,
                        received: value.length,
                    });
            }
            const [lower, upper] = value.map(v => Object.safeFrom(v));
            if (!lower.success)
                throwPgTPError(lower.error.issue);
            if (!upper.success)
                throwPgTPError(upper.error.issue);
            this._values = [lower.data, upper.data];
            this._checkEmpty();
        }
        get empty() {
            return this._values === null;
        }
        get value() {
            return this.toString();
        }
        set value(range) {
            const parsed = Range.safeFrom(range);
            if (parsed.success) {
                this._values = parsed.data.values;
                this._lower = parsed.data.lower;
                this._upper = parsed.data.upper;
            }
            else
                throw parsed.error;
        }
        get postgres() {
            return this.toString();
        }
        set postgres(range) {
            const parsed = Range.safeFrom(range);
            if (parsed.success) {
                this._values = parsed.data.values;
                this._lower = parsed.data.lower;
                this._upper = parsed.data.upper;
            }
            else
                throw parsed.error;
        }
    }
    return Range;
};
export { getRange, LowerRange, UpperRange };
//# sourceMappingURL=Range.js.map